//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
// HistoryListCtrl. emulEspaa Mod: Added by MoNKi
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// HistoryListCtrl.cpp: archivo de implementacin
//

#include "stdafx.h"
#include "emule.h"
#include "HistoryListCtrl.h"
#include "CommentDialog.h"
#include "FileInfoDialog.h"
#include "MetaDataDlg.h"
#include "ResizableLib/ResizableSheet.h"
#include "Preferences.h"
#include "KnownFileList.h"
#include "EmuleDlg.h"
#include "memdc.h"
#include "menucmds.h"
#include "IrcWnd.h"

//////////////////////////////////////////////////////////////////////////////
// CHistoryFileDetailsSheet

class CHistoryFileDetailsSheet : public CResizableSheet
{
	DECLARE_DYNAMIC(CHistoryFileDetailsSheet)

public:
	CHistoryFileDetailsSheet(CKnownFile* file);
	virtual ~CHistoryFileDetailsSheet();

protected:
	CKnownFile* m_file;
	CFileInfoDialog m_wndMediaInfo;
	CMetaDataDlg m_wndMetaData;

	static int sm_iLastActivePage;

	virtual BOOL OnInitDialog();

	DECLARE_MESSAGE_MAP()
	afx_msg void OnDestroy();
};

int CHistoryFileDetailsSheet::sm_iLastActivePage;

IMPLEMENT_DYNAMIC(CHistoryFileDetailsSheet, CResizableSheet)

BEGIN_MESSAGE_MAP(CHistoryFileDetailsSheet, CResizableSheet)
	ON_WM_DESTROY()
END_MESSAGE_MAP()

CHistoryFileDetailsSheet::CHistoryFileDetailsSheet(CKnownFile* file)
{
	m_file = file;
	m_psh.dwFlags &= ~PSH_HASHELP;
	m_psh.dwFlags |= PSH_NOAPPLYNOW;
	
	m_wndMediaInfo.m_psp.dwFlags &= ~PSP_HASHELP;
	m_wndMetaData.m_psp.dwFlags &= ~PSP_HASHELP;

	m_wndMediaInfo.SetMyfile(file);
	if (theApp.glob_prefs->IsExtControlsEnabled())
		m_wndMetaData.SetFile(file);

	AddPage(&m_wndMediaInfo);
	if (theApp.glob_prefs->IsExtControlsEnabled())
		AddPage(&m_wndMetaData);
}

CHistoryFileDetailsSheet::~CHistoryFileDetailsSheet()
{
}

void CHistoryFileDetailsSheet::OnDestroy()
{
	sm_iLastActivePage = GetActiveIndex();
	CResizableSheet::OnDestroy();
}

BOOL CHistoryFileDetailsSheet::OnInitDialog()
{		
	EnableStackedTabs(FALSE);
	BOOL bResult = CResizableSheet::OnInitDialog();
	InitWindowStyles(this);
	EnableSaveRestore(_T("HistoryFileDetailsSheet")); // call this after(!) OnInitDialog
	SetWindowText(GetResString(IDS_DETAILS) + _T(": ") + m_file->GetFileName());
	if (sm_iLastActivePage < GetPageCount())
		SetActivePage(sm_iLastActivePage);
	return bResult;
}

//////////////////////////////
// CHistoryListCtrl

#define MLC_DT_TEXT (DT_SINGLELINE | DT_NOPREFIX | DT_VCENTER | DT_END_ELLIPSIS)

IMPLEMENT_DYNAMIC(CHistoryListCtrl, CListCtrl)
CHistoryListCtrl::CHistoryListCtrl()
{
}

CHistoryListCtrl::~CHistoryListCtrl()
{
}


BEGIN_MESSAGE_MAP(CHistoryListCtrl, CMuleListCtrl)
	ON_NOTIFY_REFLECT(LVN_COLUMNCLICK, OnColumnClick)
	ON_WM_CONTEXTMENU()
END_MESSAGE_MAP()


// Controladores de mensajes de CHistoryListCtrl


void CHistoryListCtrl::Init(void)
{
	CImageList ilDummyImageList; //dummy list for getting the proper height of listview entries
	ilDummyImageList.Create(1, theApp.GetSmallSytemIconSize().cy,theApp.m_iDfltImageListColorFlags|ILC_MASK, 1, 1); 
	SetImageList(&ilDummyImageList, LVSIL_SMALL);
	ASSERT( (GetStyle() & LVS_SHAREIMAGELISTS) == 0 );
	ilDummyImageList.Detach();

	SetExtendedStyle(LVS_EX_FULLROWSELECT);
	ModifyStyle(LVS_SINGLESEL,0);
	
	InsertColumn(0,GetResString(IDS_DL_FILENAME),LVCFMT_LEFT, 260);
	InsertColumn(1,GetResString(IDS_DL_SIZE),LVCFMT_RIGHT,70);
	InsertColumn(2,GetResString(IDS_TYPE),LVCFMT_LEFT,100);
	InsertColumn(3,GetResString(IDS_FILEID),LVCFMT_LEFT, 220);
	InsertColumn(4,GetResString(IDS_DATE),LVCFMT_LEFT, 120);
	InsertColumn(5,GetResString(IDS_DOWNHISTORY_SHARED),LVCFMT_LEFT, 65);
	InsertColumn(6,GetResString(IDS_COMMENT),LVCFMT_LEFT, 260);

	Reload();

	LoadSettings(CPreferences::tableHistory);

	// Barry - Use preferred sort order from preferences
	int sortItem = theApp.glob_prefs->GetColumnSortItem(CPreferences::tableHistory);
	bool sortAscending = theApp.glob_prefs->GetColumnSortAscending(CPreferences::tableHistory);
	SetSortArrow(sortItem, sortAscending);
	// [TPT] - SLUGFILLER: multiSort - load multiple params
	for (int i = theApp.glob_prefs->GetColumnSortCount(CPreferences::tableServer); i > 0; ) {
		i--;
		sortItem = theApp.glob_prefs->GetColumnSortItem(CPreferences::tableServer, i);
		sortAscending = theApp.glob_prefs->GetColumnSortAscending(CPreferences::tableServer, i);
		SortItems(SortProc, sortItem + (sortAscending ? 0:20));
	}
	// [TPT] - SLUGFILLER: multiSort
}

void CHistoryListCtrl::AddFile(CKnownFile* toadd){
	uint32 itemnr = GetItemCount();

	LVFINDINFO info;
	info.flags = LVFI_PARAM;
	info.lParam = (LPARAM)toadd;
	int nItem = FindItem(&info);
	if(nItem == -1){
		if(!toadd->IsShared() || (toadd->IsShared() && theApp.glob_prefs->GetShowSharedInHistory()))
			InsertItem(LVIF_PARAM|LVIF_TEXT,itemnr,toadd->GetFileName(),0,0,0,(LPARAM)toadd);
	}
}

void CHistoryListCtrl::Reload(void)
{
	CKnownFile * cur_file;
	CArray<CKnownFile *, CKnownFile *> *files;

	SetRedraw(false);

	DeleteAllItems();
	
	if(theApp.glob_prefs->GetShowSharedInHistory())
		files=theApp.knownfiles;
	else
		files=theApp.knownfiles->GetDownloadedFiles();

	int count = files->GetCount();
	for(int i=0; i<count; i++){
		if(cur_file = files->GetAt(i))
			InsertItem(LVIF_PARAM|LVIF_TEXT,GetItemCount(),cur_file->GetFileName(),0,0,0,(LPARAM)cur_file);
	}

    if(!theApp.glob_prefs->GetShowSharedInHistory())
		delete files;

	SetRedraw(true);
}

void CHistoryListCtrl::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct) {
	if( !theApp.emuledlg->IsRunning() )
		return;
	if( !lpDrawItemStruct->itemData)
		return;

	//set up our ficker free drawing
	CRect rcItem(lpDrawItemStruct->rcItem);
	CDC *oDC = CDC::FromHandle(lpDrawItemStruct->hDC);
	COLORREF crOldDCBkColor = oDC->SetBkColor(m_crWindow);
	CMemDC pDC(oDC, &rcItem);	
	CFont *pOldFont = pDC->SelectObject(GetFont());
	COLORREF crOldTextColor;
	CKnownFile* file = (CKnownFile*)lpDrawItemStruct->itemData;
	
	if(m_bCustomDraw)
		crOldTextColor = pDC->SetTextColor(m_lvcd.clrText);
	else
		crOldTextColor = pDC->SetTextColor(m_crWindowText);

	int iOffset = pDC->GetTextExtent(_T(" "), 1 ).cx*2;
	int iItem = lpDrawItemStruct->itemID;
	CHeaderCtrl *pHeaderCtrl = GetHeaderCtrl();

	//gets the item image and state info
	LV_ITEM lvi;
	lvi.mask = LVIF_IMAGE | LVIF_STATE;
	lvi.iItem = iItem;
	lvi.iSubItem = 0;
	lvi.stateMask = LVIS_DROPHILITED | LVIS_FOCUSED | LVIS_SELECTED;
	GetItem(&lvi);

	//see if the item be highlighted
	BOOL bHighlight = ((lvi.state & LVIS_DROPHILITED) || (lvi.state & LVIS_SELECTED));
	BOOL bCtrlFocused = ((GetFocus() == this) || (GetStyle() & LVS_SHOWSELALWAYS));

	//get rectangles for drawing
	CRect rcBounds, rcLabel, rcIcon;
	GetItemRect(iItem, rcBounds, LVIR_BOUNDS);
	GetItemRect(iItem, rcLabel, LVIR_LABEL);
	GetItemRect(iItem, rcIcon, LVIR_ICON);
	CRect rcCol(rcBounds);

	//the label!
	CString sLabel = GetItemText(iItem, 0);
	//labels are offset by a certain amount
	//this offset is related to the width of a space character
	CRect rcHighlight;
	CRect rcWnd;

	//should I check (GetExtendedStyle() & LVS_EX_FULLROWSELECT) ?
	rcHighlight.top    = rcBounds.top;
	rcHighlight.bottom = rcBounds.bottom;
	rcHighlight.left   = rcBounds.left  + 1;
	rcHighlight.right  = rcBounds.right - 1;

	COLORREF crOldBckColor;
	//draw the background color
	if(bHighlight) 
	{
		if(bCtrlFocused) 
		{
			pDC->FillRect(rcHighlight, &CBrush(m_crHighlight));
			crOldBckColor = pDC->SetBkColor(m_crHighlight);
		} 
		else 
		{
			pDC->FillRect(rcHighlight, &CBrush(m_crNoHighlight));
			crOldBckColor = pDC->SetBkColor(m_crNoHighlight);
		}
	} 
	else 
	{
		pDC->FillRect(rcHighlight, &CBrush(m_crWindow));
		crOldBckColor = pDC->SetBkColor(GetBkColor());
	}

	//update column
	rcCol.right = rcCol.left + GetColumnWidth(0);

	//draw the item's icon
	int iImage = theApp.GetFileTypeSystemImageIdx( file->GetFileName() );
	if (theApp.GetSystemImageList() != NULL)
		::ImageList_Draw(theApp.GetSystemImageList(), iImage, pDC, rcCol.left + 4, rcCol.top, ILD_NORMAL|ILD_TRANSPARENT);

	//draw item label (column 0)
	sLabel = file->GetFileName();
	rcLabel.left += 16;
	rcLabel.left += iOffset / 2;
	rcLabel.right -= iOffset;
	pDC->DrawText(sLabel, -1, rcLabel, MLC_DT_TEXT | DT_LEFT | DT_NOCLIP);

	//draw labels for remaining columns
	LV_COLUMN lvc;
	lvc.mask = LVCF_FMT | LVCF_WIDTH;
	rcBounds.right = rcHighlight.right > rcBounds.right ? rcHighlight.right : rcBounds.right;

	int iCount = pHeaderCtrl->GetItemCount();
	for(int iCurrent = 0; iCurrent < iCount; iCurrent++) 
	{
		int iColumn = pHeaderCtrl->OrderToIndex(iCurrent);

		if(iColumn == 0)
			continue;

		GetColumn(iColumn, &lvc);
		//don't draw anything with 0 width
		if(lvc.cx == 0)
			continue;

		rcCol.left = rcCol.right;
		rcCol.right += lvc.cx;

		switch(iColumn){
			case 1:
				sLabel = CastItoXBytes(file->GetFileSize());
				break;
			case 2:
				sLabel = file->GetFileType();
				break;
			case 3:
				sLabel = EncodeBase16(file->GetFileHash(),16);
				break;
			case 4:
				sLabel = file->GetCFileDate().Format("%x %X");
				break;
			case 5:
				if (file->IsShared())
					sLabel=GetResString(IDS_YES);
				else
					sLabel=GetResString(IDS_NO);
				break;
			case 6:
				sLabel = file->GetFileComment();
				break;
		}
		if (sLabel.GetLength() == 0)
			continue;

		//get the text justification
		UINT nJustify = DT_LEFT;
		switch(lvc.fmt & LVCFMT_JUSTIFYMASK) 
		{
			case LVCFMT_RIGHT:
				nJustify = DT_RIGHT;
				break;
			case LVCFMT_CENTER:
				nJustify = DT_CENTER;
				break;
			default:
				break;
		}

		rcLabel = rcCol;
		rcLabel.left += iOffset;
		rcLabel.right -= iOffset;

		pDC->DrawText(sLabel, -1, rcLabel, MLC_DT_TEXT | nJustify);
	}

	//draw focus rectangle if item has focus
	if((lvi.state & LVIS_FOCUSED) && (bCtrlFocused || (lvi.state & LVIS_SELECTED))) 
	{
		if(!bCtrlFocused || !(lvi.state & LVIS_SELECTED))
			pDC->FrameRect(rcHighlight, &CBrush(m_crNoFocusLine));
		else
			pDC->FrameRect(rcHighlight, &CBrush(m_crFocusLine));
	}

	pDC->Flush();
	//restore old font
	pDC->SelectObject(pOldFont);
	pDC->SetTextColor(crOldTextColor);
	pDC->SetBkColor(crOldBckColor);
	oDC->SetBkColor(crOldDCBkColor);
}

void CHistoryListCtrl::OnColumnClick( NMHDR* pNMHDR, LRESULT* pResult){
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;

	// Barry - Store sort order in preferences
	// Determine ascending based on whether already sorted on this column
	int sortItem = theApp.glob_prefs->GetColumnSortItem(CPreferences::tableHistory);
	bool m_oldSortAscending = theApp.glob_prefs->GetColumnSortAscending(CPreferences::tableHistory);
	bool sortAscending = (sortItem != pNMListView->iSubItem) ? true : !m_oldSortAscending;

	// Item is column clicked
	sortItem = pNMListView->iSubItem;

	// Save new preferences
	theApp.glob_prefs->SetColumnSortItem(CPreferences::tableHistory, sortItem);
	theApp.glob_prefs->SetColumnSortAscending(CPreferences::tableHistory, sortAscending);

	// Sort table
	SetSortArrow(sortItem, sortAscending);
	SortItems(SortProc, sortItem + (sortAscending ? 0:20));

	*pResult = 0;
}

int CHistoryListCtrl::SortProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort){
	CKnownFile* item1 = (CKnownFile*)lParam1;
	CKnownFile* item2 = (CKnownFile*)lParam2;	
	switch(lParamSort){
		case 0: //filename asc
			return _tcsicmp(item1->GetFileName(),item2->GetFileName());
		case 20: //filename desc
			return _tcsicmp(item2->GetFileName(),item1->GetFileName());

		case 1: //filesize asc
			return item1->GetFileSize()==item2->GetFileSize()?0:(item1->GetFileSize()>item2->GetFileSize()?1:-1);

		case 21: //filesize desc
			return item1->GetFileSize()==item2->GetFileSize()?0:(item2->GetFileSize()>item1->GetFileSize()?1:-1);

		case 2: //filetype asc
			return item1->GetFileType().CompareNoCase(item2->GetFileType());
		case 22: //filetype desc
			return item2->GetFileType().CompareNoCase(item1->GetFileType());

		case 3: //file ID
			return memcmp(item1->GetFileHash(),item2->GetFileHash(),16);
		case 23:
			return memcmp(item2->GetFileHash(),item1->GetFileHash(),16);

		case 4: //date
			return CompareUnsigned(item1->GetFileDate(),item2->GetFileDate());
		case 24:
			return CompareUnsigned(item2->GetFileDate(),item1->GetFileDate());

		case 5: //Shared?
			return item1->IsShared()==item2->IsShared() ? 0 : (item1->IsShared() && !item2->IsShared() ? 1 : -1);
		case 25:
			return item1->IsShared()==item2->IsShared() ? 0 : (item2->IsShared() && !item1->IsShared() ? 1 : -1);

		case 6: //comment
			return _tcsicmp(item1->GetFileComment(),item2->GetFileComment());
		case 26:
			return _tcsicmp(item2->GetFileComment(),item1->GetFileComment());

		default: 
			return 0;
	}
}

void CHistoryListCtrl::Localize() {
	CHeaderCtrl* pHeaderCtrl = GetHeaderCtrl();
	HDITEM hdi;
	hdi.mask = HDI_TEXT;
	CString strRes;

	for(int i=0; i<=6;i++){
		switch(i){
			case 0:
				strRes = GetResString(IDS_DL_FILENAME);
				break;
			case 1:
				strRes = GetResString(IDS_DL_SIZE);
				break;
			case 2:
				strRes = GetResString(IDS_TYPE);
				break;
			case 3:
				strRes = GetResString(IDS_FILEID);
				break;
			case 4:
				strRes = GetResString(IDS_DATE);
				break;
			case 5:
				strRes = GetResString(IDS_DOWNHISTORY_SHARED);
				break;
			case 6:
				strRes = GetResString(IDS_COMMENT);
				break;
			default:
				strRes = "No Text!!";
		}

		hdi.pszText = strRes.GetBuffer();
		pHeaderCtrl->SetItem(i, &hdi);
		strRes.ReleaseBuffer();
	}

	CreateMenues();
}

void CHistoryListCtrl::CreateMenues()
{
	if (m_HistoryOpsMenu) VERIFY( m_HistoryOpsMenu.DestroyMenu() );
	if (m_HistoryMenu) VERIFY(m_HistoryMenu.DestroyMenu());

	m_HistoryOpsMenu.CreateMenu();
	m_HistoryOpsMenu.AppendMenu(MF_STRING,MP_CLEARHISTORY,GetResString(IDS_DOWNHISTORY_CLEAR));
	m_HistoryOpsMenu.AppendMenu(MF_STRING,MP_MERGEHISTORY,GetResString(IDS_DOWNHISTORY_MERGE));

	m_HistoryMenu.CreatePopupMenu();
	m_HistoryMenu.AddMenuTitle(GetResString(IDS_DOWNHISTORY));
	// [TPT] - Menu Color
	m_HistoryMenu.SetColor(RGB(255,000,000)); 
	m_HistoryMenu.SetGradientColor(RGB(255,128,0));
	// [TPT] - Menu Color
	m_HistoryMenu.AppendMenu(MF_STRING,MP_OPEN, GetResString(IDS_OPENFILE));
	
	m_HistoryMenu.AppendMenu(MF_STRING,MP_CMT, GetResString(IDS_CMT_ADD)); 
	m_HistoryMenu.AppendMenu(MF_STRING,MP_DETAIL, GetResString(IDS_SHOWDETAILS));
	m_HistoryMenu.AppendMenu(MF_STRING|MF_SEPARATOR); 

	m_HistoryMenu.AppendMenu(MF_STRING,MP_VIEWSHAREDFILES,GetResString(IDS_DOWNHISTORY_SHOWSHARED));
	m_HistoryMenu.AppendMenu(MF_STRING|MF_SEPARATOR); 

	m_HistoryMenu.AppendMenu(MF_STRING,MP_GETED2KLINK, GetResString(IDS_DL_LINK1));
	m_HistoryMenu.AppendMenu(MF_STRING,MP_GETHTMLED2KLINK, GetResString(IDS_DL_LINK2));
	m_HistoryMenu.AppendMenu(MF_STRING, MP_GETBBCODEED2KLINK, GetResString(IDS_CREATEBBCODELINK));	// emulEspaa: added by Announ [Announ: -Copy BBCode ed2k links-]
	m_HistoryMenu.AppendMenu(MF_STRING|MF_SEPARATOR); 
	
	m_HistoryMenu.AppendMenu(MF_STRING,MP_REMOVESELECTED, GetResString(IDS_DOWNHISTORY_REMOVE));
	m_HistoryMenu.AppendMenu(MF_STRING|MF_SEPARATOR); 

	//This menu option is is for testing..
	if(theApp.emuledlg->ircwnd->IsConnected()){
		m_HistoryMenu.AppendMenu(MF_STRING,Irc_SetSendLink,GetResString(IDS_IRC_ADDLINKTOIRC));
		m_HistoryMenu.AppendMenu(MF_STRING|MF_SEPARATOR); 
	}

	m_HistoryMenu.AppendMenu(MF_POPUP,(UINT_PTR)m_HistoryOpsMenu.m_hMenu, GetResString(IDS_DOWNHISTORY_ACTIONS));
	m_HistoryMenu.AppendMenu(MF_STRING|MF_SEPARATOR); 
}

void CHistoryListCtrl::OnContextMenu(CWnd* pWnd, CPoint point)
{
	CKnownFile* file = NULL;

	if (GetSelectionMark()!=-1) file=(CKnownFile*)GetItemData(GetSelectionMark());

	if(file && file->IsShared()){
		m_HistoryMenu.EnableMenuItem(MP_OPEN, MF_ENABLED);
		m_HistoryMenu.EnableMenuItem(MP_REMOVESELECTED, MF_GRAYED);
		m_HistoryMenu.EnableMenuItem(MP_DETAIL, MF_ENABLED);
	}
	else {
		m_HistoryMenu.EnableMenuItem(MP_OPEN, MF_GRAYED);
		m_HistoryMenu.EnableMenuItem(MP_DETAIL, MF_GRAYED);
		if(file && GetSelectedCount()>0)
            m_HistoryMenu.EnableMenuItem(MP_REMOVESELECTED, MF_ENABLED);
        else
		    m_HistoryMenu.EnableMenuItem(MP_REMOVESELECTED, MF_GRAYED);
    }

	if(theApp.glob_prefs->GetShowSharedInHistory())
		m_HistoryMenu.CheckMenuItem(MP_VIEWSHAREDFILES, MF_CHECKED);
	else
		m_HistoryMenu.CheckMenuItem(MP_VIEWSHAREDFILES, MF_UNCHECKED);

	int counter;
	CMenu m_Web;
	m_Web.CreateMenu();
	UpdateURLMenu(m_Web,counter);
	UINT flag2;
	flag2=(counter==0) ? MF_GRAYED:MF_STRING;
	m_HistoryMenu.AppendMenu(flag2|MF_POPUP,(UINT_PTR)m_Web.m_hMenu, GetResString(IDS_WEBSERVICES) );

	m_HistoryMenu.TrackPopupMenu(TPM_LEFTALIGN |TPM_RIGHTBUTTON,point.x,point.y,this);

	m_HistoryMenu.RemoveMenu(m_HistoryMenu.GetMenuItemCount()-1,MF_BYPOSITION);
	VERIFY( m_Web.DestroyMenu() );

}

void CHistoryListCtrl::OnNMDblclk(NMHDR *pNMHDR, LRESULT *pResult)
{
	int iSel = GetSelectionMark();
	if (iSel != -1) {
		CKnownFile* file = (CKnownFile*)GetItemData(iSel);
		if (file){
			if (GetKeyState(VK_MENU) & 0x8000){
				CHistoryFileDetailsSheet sheet(file);
				sheet.DoModal();
			}
			else if (file->IsShared())
				OpenFile(file);
		}
	}
	*pResult = 0;
}

BOOL CHistoryListCtrl::OnCommand(WPARAM wParam, LPARAM lParam)
{
	UINT selectedCount = this->GetSelectedCount(); 
	int iSel = GetSelectionMark();

	if (selectedCount>0){
		CKnownFile* file = (CKnownFile*)GetItemData(iSel);
		if (wParam>=MP_WEBURL && wParam<=MP_WEBURL+256) {
			RunURL(file, theApp.webservices.GetAt(wParam-MP_WEBURL) );
		}

		switch (wParam){
			case Irc_SetSendLink:
			{
				theApp.emuledlg->ircwnd->SetSendFileString(CreateED2kLink(file));
				break;
			}
			case MP_GETED2KLINK:{
				if(selectedCount > 1)
				{
					CString str;
					POSITION pos = this->GetFirstSelectedItemPosition();
					while( pos != NULL )
					{
						file = (CKnownFile*)this->GetItemData(GetNextSelectedItem(pos));
						str.Append(CreateED2kLink(file) + "\n");
					}

					theApp.CopyTextToClipboard(str);
					//AfxMessageBox(GetResString(IDS_COPIED2CB) + str);
					break; 
				}
				theApp.CopyTextToClipboard(CreateED2kLink(file));
				break;
			}
			case MP_GETHTMLED2KLINK:
				if(selectedCount > 1)
				{
					CString str;
					POSITION pos = this->GetFirstSelectedItemPosition();
					while( pos != NULL )
					{
						file = (CKnownFile*)this->GetItemData(GetNextSelectedItem(pos));
						str += CreateHTMLED2kLink(file) + "\n"; 
					}
					theApp.CopyTextToClipboard(str);
					//AfxMessageBox(GetResString(IDS_COPIED2CB) + str);
					break; 
				} 
				theApp.CopyTextToClipboard(CreateHTMLED2kLink(file));
				break;
			// emulEspaa: added by Announ [Announ: -Copy BBCode ed2k links-]
			case MP_GETBBCODEED2KLINK:
			{
				if(selectedCount > 1)
				{
					CString str;
					POSITION pos = this->GetFirstSelectedItemPosition();
					while( pos != NULL )
					{
						file = (CKnownFile*)this->GetItemData(GetNextSelectedItem(pos));
						str += theApp.CreateBBCodeED2kLink(file) + "\n"; 
					}
					theApp.CopyTextToClipboard(str);
					//AfxMessageBox(GetResString(IDS_COPIED2CB) + str);
					break; 
				} 
				CString strLink = theApp.CreateBBCodeED2kLink(file);
				if (!strLink.IsEmpty())
					theApp.CopyTextToClipboard(strLink);
				break;
			}
			// End emulEspaa			
			case MP_OPEN:
				if(file->IsShared())
					OpenFile(file);
				break; 
			//For Comments 
			case MP_CMT: 
            { 
				ShowComments(iSel);
                break; 
            } 
            //*END Comments*//
			case MPG_ALTENTER:
			case MP_DETAIL:
				if (file){
					CHistoryFileDetailsSheet sheet(file);
					sheet.DoModal();
				}
				break;
			case MP_REMOVESELECTED:
				{
					UINT i, uSelectedCount = GetSelectedCount();
					int  nItem = -1;

					if (uSelectedCount > 1)
					{
						if(MessageBox(GetResString(IDS_DOWNHISTORY_REMOVE_QUESTION_MULTIPLE),NULL,MB_YESNO) == IDYES){
							for (i=0;i < uSelectedCount;i++)
							{
								nItem = GetNextItem(nItem, LVNI_SELECTED);
								ASSERT(nItem != -1);
								CKnownFile *item_File = (CKnownFile *)GetItemData(nItem);
								if(item_File && !item_File->IsShared()){
									RemoveFile(item_File);
									nItem--;
								}
							}
						}
					}
					else
					{
						if(file){
							CString msg;
							msg.Format(GetResString(IDS_DOWNHISTORY_REMOVE_QUESTION),file->GetFileName());
							if(MessageBox(msg,NULL,MB_YESNO) == IDYES)
								RemoveFile(file);
						}
					}
				}
				break;
		}
	}

	switch(wParam){
		case MP_CLEARHISTORY:
			ClearHistory();
			break;
		case MP_MERGEHISTORY:
			RemoveDuplicates();
			break;
		case MP_VIEWSHAREDFILES:
			theApp.glob_prefs->SetShowSharedInHistory(!theApp.glob_prefs->GetShowSharedInHistory());
			Reload();
			break;
	}

	return true;
}

void CHistoryListCtrl::ShowComments(int index) {
	CKnownFile* file = (CKnownFile*)GetItemData(index);
    CCommentDialog dialog(file); 
    dialog.DoModal(); 
}

void CHistoryListCtrl::OpenFile(CKnownFile* file){
	char* buffer = new char[MAX_PATH];
	_snprintf(buffer,MAX_PATH,"%s\\%s",file->GetPath(),file->GetFileName());
	AddLogLine( false, "%s\\%s",file->GetPath(),file->GetFileName());
	ShellOpenFile(buffer, NULL);
	delete[] buffer;
}

void CHistoryListCtrl::RemoveFile(CKnownFile *toRemove) {
	if(toRemove->IsShared())
		return;

	if(theApp.knownfiles->RemoveKnownFile(toRemove)){
		LVFINDINFO info;
		info.flags = LVFI_PARAM;
		info.lParam = (LPARAM)toRemove;
		int nItem = FindItem(&info);
		if(nItem != -1)
			DeleteItem(nItem);
	}
}

void CHistoryListCtrl::ClearHistory() {
	if(MessageBox(GetResString(IDS_DOWNHISTORY_CLEAR_QUESTION),GetResString(IDS_DOWNHISTORY),MB_YESNO)==IDYES)
	{
		theApp.knownfiles->ClearHistory();
		Reload();
	}
}

void CHistoryListCtrl::RemoveDuplicates() {
	theApp.knownfiles->RemoveDuplicates();
	Reload();
}